@@ -3,6 +3,7 @@
 from decimal import Decimal
 import datetime
 import operator
+from collections import defaultdict
 from functools import wraps
 
 from dateutil.relativedelta import relativedelta
@@ -198,6 +199,7 @@ class Type(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
         pool = Pool()
         Account = pool.get('account.account')
         GeneralLedger = pool.get('account.general_ledger.account')
+        context = Transaction().context
 
         res = {}
         for type_ in types:
@@ -210,10 +212,13 @@ class Type(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
         for type_ in childs:
             type_sum[type_.id] = Decimal('0.0')
 
-        start_period_ids = GeneralLedger.get_period_ids('start_%s' % name)
-        end_period_ids = GeneralLedger.get_period_ids('end_%s' % name)
-        period_ids = list(
-            set(end_period_ids).difference(set(start_period_ids)))
+        if context.get('start_period') or context.get('end_period'):
+            start_period_ids = GeneralLedger.get_period_ids('start_%s' % name)
+            end_period_ids = GeneralLedger.get_period_ids('end_%s' % name)
+            period_ids = list(
+                set(end_period_ids).difference(set(start_period_ids)))
+        else:
+            period_ids = None
 
         with Transaction().set_context(periods=period_ids):
             accounts = Account.search([
@@ -932,7 +937,7 @@ class Account(ActivePeriodMixin, tree(), ModelSQL, ModelView):
                         values[name][account.id] += getattr(deferral, name)
         else:
             with Transaction().set_context(fiscalyear=fiscalyear.id,
-                    date=None, periods=None):
+                    date=None, periods=None, from_date=None, to_date=None):
                 previous_result = func(accounts, names)
             for name in names:
                 vals = values[name]
@@ -1179,8 +1184,39 @@ class AccountDeferral(ModelSQL, ModelView):
     def default_amount_second_currency(cls):
         return Decimal(0)
 
-    def get_balance(self, name):
-        return self.debit - self.credit
+    @classmethod
+    def get_balance(cls, deferrals, name):
+        pool = Pool()
+        Account = pool.get('account.account')
+        cursor = Transaction().connection.cursor()
+
+        table = cls.__table__()
+        table_child = cls.__table__()
+        account = Account.__table__()
+        account_child = Account.__table__()
+        balances = defaultdict(Decimal)
+
+        for sub_deferrals in grouped_slice(deferrals):
+            red_sql = reduce_ids(table.id, [d.id for d in sub_deferrals])
+            cursor.execute(*table
+                .join(account, condition=table.account == account.id)
+                .join(account_child,
+                    condition=(account_child.left >= account.left)
+                    & (account_child.right <= account.right))
+                .join(table_child,
+                    condition=(table_child.account == account_child.id)
+                    & (table_child.fiscalyear == table.fiscalyear))
+                .select(
+                    table.id,
+                    Sum(table_child.debit - table_child.credit),
+                    where=red_sql,
+                    group_by=table.id))
+            balances.update(dict(cursor))
+
+        for id_, balance in balances.items():
+            if not isinstance(balance, Decimal):
+                balances[id_] = Decimal(str(balance))
+        return balances
 
     def get_currency(self, name):
         return self.account.currency.id
@@ -1384,8 +1420,16 @@ class GeneralLedgerAccount(ActivePeriodMixin, ModelSQL, ModelView):
         pool = Pool()
         Account = pool.get('account.account')
 
-        period_ids = cls.get_period_ids(name)
-        from_date, to_date = cls.get_dates(name)
+        period_ids, from_date, to_date = None, None, None
+        context = Transaction().context
+        if context.get('start_period') or context.get('end_period'):
+            period_ids = cls.get_period_ids(name)
+        elif context.get('from_date') or context.get('end_date'):
+            from_date, to_date = cls.get_dates(name)
+        else:
+            if name.startswith('start_'):
+                period_ids = []
+
         with Transaction().set_context(
                 periods=period_ids,
                 from_date=from_date, to_date=to_date):
@@ -1427,16 +1471,29 @@ class GeneralLedgerAccount(ActivePeriodMixin, ModelSQL, ModelView):
             if operator_(getattr(a, fname), operand)]
         return [('id', 'in', ids)]
 
+    @classmethod
+    def _debit_credit_context(cls):
+        period_ids, from_date, to_date = None, None, None
+        context = Transaction().context
+        if context.get('start_period') or context.get('end_period'):
+            start_period_ids = set(cls.get_period_ids('start_balance'))
+            end_period_ids = set(cls.get_period_ids('end_balance'))
+            period_ids = list(end_period_ids.difference(start_period_ids))
+        elif context.get('from_date') or context.get('end_date'):
+            _, from_date = cls.get_dates('start_balance')
+            _, to_date = cls.get_dates('end_balance')
+        return {
+            'periods': period_ids,
+            'from_date': from_date,
+            'to_date': to_date,
+            }
+
     @classmethod
     def get_debit_credit(cls, records, name):
         pool = Pool()
         Account = pool.get('account.account')
 
-        start_period_ids = cls.get_period_ids('start_%s' % name)
-        end_period_ids = cls.get_period_ids('end_%s' % name)
-        periods_ids = list(
-            set(end_period_ids).difference(set(start_period_ids)))
-        with Transaction().set_context(periods=periods_ids):
+        with Transaction().set_context(cls._debit_credit_context()):
             accounts = Account.browse(records)
         return {a.id: getattr(a, name) for a in accounts}
 
@@ -1445,11 +1502,7 @@ class GeneralLedgerAccount(ActivePeriodMixin, ModelSQL, ModelView):
         pool = Pool()
         Account = pool.get('account.account')
 
-        start_period_ids = cls.get_period_ids('start_%s' % name)
-        end_period_ids = cls.get_period_ids('end_%s' % name)
-        periods_ids = list(
-            set(end_period_ids).difference(set(start_period_ids)))
-        with Transaction().set_context(periods=periods_ids):
+        with Transaction().set_context(cls._debit_credit_context()):
             accounts = Account.search([], order=[])
 
         _, operator_, operand = domain
@@ -1485,27 +1538,42 @@ class GeneralLedgerAccountContext(ModelView):
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
             ('start_date', '<=', (Eval('end_period'), 'start_date')),
-            ], depends=['fiscalyear', 'end_period'])
+            ],
+        states={
+            'invisible': Eval('from_date', False) | Eval('to_date', False),
+            },
+        depends=['fiscalyear', 'end_period', 'from_date', 'to_date'])
     end_period = fields.Many2One('account.period', 'End Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
             ('start_date', '>=', (Eval('start_period'), 'start_date'))
             ],
-        depends=['fiscalyear', 'start_period'])
+        states={
+            'invisible': Eval('from_date', False) | Eval('to_date', False),
+            },
+        depends=['fiscalyear', 'start_period', 'from_date', 'to_date'])
     from_date = fields.Date("From Date",
         domain=[
             If(Eval('to_date') & Eval('from_date'),
                 ('from_date', '<=', Eval('to_date')),
                 ()),
             ],
-        depends=['to_date'])
+        states={
+            'invisible': (Eval('start_period', False)
+                | Eval('end_period', False)),
+            },
+        depends=['to_date', 'start_period', 'end_period'])
     to_date = fields.Date("To Date",
         domain=[
             If(Eval('from_date') & Eval('to_date'),
                 ('to_date', '>=', Eval('from_date')),
                 ()),
             ],
-        depends=['from_date'])
+        states={
+            'invisible': (Eval('start_period', False)
+                | Eval('end_period', False)),
+            },
+        depends=['from_date', 'start_period', 'end_period'])
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
 
@@ -1551,6 +1619,26 @@ class GeneralLedgerAccountContext(ModelView):
                 and self.end_period.fiscalyear != self.fiscalyear):
             self.end_period = None
 
+    @fields.depends('start_period')
+    def on_change_start_period(self):
+        if self.start_period:
+            self.from_date = self.to_date = None
+
+    @fields.depends('end_period')
+    def on_change_end_period(self):
+        if self.end_period:
+            self.from_date = self.to_date = None
+
+    @fields.depends('from_date')
+    def on_change_from_date(self):
+        if self.from_date:
+            self.start_period = self.end_period = None
+
+    @fields.depends('to_date')
+    def on_change_to_date(self):
+        if self.to_date:
+            self.start_period = self.end_period = None
+
 
 class GeneralLedgerLine(ModelSQL, ModelView):
     'General Ledger Line'
@@ -1630,10 +1718,7 @@ class GeneralLedgerLine(ModelSQL, ModelView):
             else:
                 column = Column(line, fname).as_(fname)
             columns.append(column)
-        start_period_ids = set(LedgerAccount.get_period_ids('start_balance'))
-        end_period_ids = set(LedgerAccount.get_period_ids('end_balance'))
-        period_ids = list(end_period_ids.difference(start_period_ids))
-        with Transaction().set_context(periods=period_ids):
+        with Transaction().set_context(LedgerAccount._debit_credit_context()):
             line_query, fiscalyear_ids = Line.query_get(line)
         return line.join(move, condition=line.move == move.id
             ).join(account, condition=line.account == account.id
@@ -1787,27 +1872,41 @@ class IncomeStatementContext(ModelView):
             ('fiscalyear', '=', Eval('fiscalyear')),
             ('start_date', '<=', (Eval('end_period'), 'start_date'))
             ],
-        depends=['end_period', 'fiscalyear'])
+        states={
+            'invisible': Eval('from_date', False) | Eval('to_date', False),
+            },
+        depends=['end_period', 'fiscalyear', 'from_date', 'to_date'])
     end_period = fields.Many2One('account.period', 'End Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
             ('start_date', '>=', (Eval('start_period'), 'start_date')),
             ],
-        depends=['start_period', 'fiscalyear'])
+        states={
+            'invisible': Eval('from_date', False) | Eval('to_date', False),
+            },
+        depends=['start_period', 'fiscalyear', 'from_date', 'to_date'])
     from_date = fields.Date("From Date",
         domain=[
             If(Eval('to_date') & Eval('from_date'),
                 ('from_date', '<=', Eval('to_date')),
                 ()),
             ],
-        depends=['to_date'])
+        states={
+            'invisible': (
+                Eval('start_period', False) | Eval('end_period', False)),
+            },
+        depends=['to_date', 'start_period', 'end_period'])
     to_date = fields.Date("To Date",
         domain=[
             If(Eval('from_date') & Eval('to_date'),
                 ('to_date', '>=', Eval('from_date')),
                 ()),
             ],
-        depends=['from_date'])
+        states={
+            'invisible': (
+                Eval('start_period', False) | Eval('end_period', False)),
+            },
+        depends=['from_date', 'start_period', 'end_period'])
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
     comparison = fields.Boolean('Comparison')
@@ -1877,10 +1976,34 @@ class IncomeStatementContext(ModelView):
     def default_comparison(cls):
         return False
 
-    @fields.depends('fiscalyear')
+    @fields.depends('fiscalyear', 'start_period', 'end_period')
     def on_change_fiscalyear(self):
-        self.start_period = None
-        self.end_period = None
+        if (self.start_period
+                and self.start_period.fiscalyear != self.fiscalyear):
+            self.start_period = None
+        if (self.end_period
+                and self.end_period.fiscalyear != self.fiscalyear):
+            self.end_period = None
+
+    @fields.depends('start_period')
+    def on_change_start_period(self):
+        if self.start_period:
+            self.from_date = self.to_date = None
+
+    @fields.depends('end_period')
+    def on_change_end_period(self):
+        if self.end_period:
+            self.from_date = self.to_date = None
+
+    @fields.depends('from_date')
+    def on_change_from_date(self):
+        if self.from_date:
+            self.start_period = self.end_period = None
+
+    @fields.depends('to_date')
+    def on_change_to_date(self):
+        if self.to_date:
+            self.start_period = self.end_period = None
 
     @classmethod
     def view_attributes(cls):
