@@ -82,6 +82,7 @@ class Move(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Move, cls).__setup__()
+        cls.create_date.select = True
         cls._check_modify_exclude = []
         cls._order.insert(0, ('date', 'DESC'))
         cls._order.insert(1, ('number', 'DESC'))
@@ -136,9 +137,6 @@ class Move(ModelSQL, ModelView):
         table = cls.__table_handler__(module_name)
         table.index_action(['journal', 'period'], 'add')
 
-        # Add index on create_date
-        table.index_action('create_date', action='add')
-
     @classmethod
     def order_post_number(cls, tables):
         table, _ = tables[None]
@@ -958,7 +956,9 @@ class Line(ModelSQL, ModelView):
                     ('start_date', '<=', date),
                     ('end_date', '>=', date),
                     ('company', '=', company),
-                    ], limit=1)
+                    ],
+                order=[('start_date', 'DESC')],
+                limit=1)
             if fiscalyears:
                 fiscalyear_id = fiscalyears[0].id
             else:
@@ -1376,15 +1376,25 @@ class OpenAccount(Wizard):
     open_ = StateAction('account.act_move_line_form')
 
     def do_open_(self, action):
-        FiscalYear = Pool().get('account.fiscalyear')
+        pool = Pool()
+        FiscalYear = pool.get('account.fiscalyear')
+        context = Transaction().context
 
-        if not Transaction().context.get('fiscalyear'):
+        date = context.get('date')
+        fiscalyear = context.get('fiscalyear')
+        if date:
+            fiscalyears = FiscalYear.search([
+                    ('start_date', '<=', date),
+                    ('end_date', '>=', date),
+                    ],
+                order=[('start_date', 'DESC')],
+                limit=1)
+        elif fiscalyear:
+            fiscalyears = [FiscalYear(fiscalyear)]
+        else:
             fiscalyears = FiscalYear.search([
                     ('state', '=', 'open'),
                     ])
-        else:
-            fiscalyears = [FiscalYear(Transaction().context['fiscalyear'])]
-
         periods = [p for f in fiscalyears for p in f.periods]
 
         action['pyson_domain'] = [
@@ -1593,7 +1603,7 @@ class Reconcile(Wizard):
             party = parties.pop()
             self.show.party = party
             self.show.parties = parties
-            return party
+            return party,
 
         if getattr(self.show, 'accounts', None) is None:
             self.show.accounts = self.get_accounts()
